/**
 * utils.js
 *
 * Copyright (C) 2013 by Florian Holzapfel
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
**/
function get_arr_key(object) {
	for(key in object) {
		if(key !== '$') {
			return key;
		}
	}
	return null;
}

var parse_xml = function(object, xml_object) {
	for(var key in xml_object) {
		var o = xml_object[key][0];
		key = key.replace('-', '_');
		
		if(typeof(o) == 'string') {
			object[key] = o;
		} else if(typeof(o) == 'object') {
			if(typeof(o.$) == 'object') {
				if(o.$.nil === 'true') {
					object[key] = null;
				} else if(o.$.type === 'array') {
					object[key] = [];

					var arr_key = get_arr_key(o);
					if(arr_key) {
						for(var i = 0; i < o[arr_key].length; ++i) {
							object[key].push({});
							parse_xml(object[key][i], o[arr_key][i]);
						}
					}
				} else if(o.$.type === 'datetime') {
					object[key] = new Date(o._);
				} else if(o.$.type === 'integer') {
					object[key] = parseInt(o._);
				}
			} else {
				object[key] = { };
				parse_xml(object[key], o);
			}
		}
	}
}

module.exports.parse_xml = parse_xml;
